from typing import List
import importlib.resources


def get_data(folder: str, filename: str) -> List[str]:
    words = []

    with importlib.resources.path('randword', 'data') as data_path:
        filepath = data_path / folder / f'{filename}.txt'

        with open(filepath, 'r', encoding='utf-8') as pos_file:
            pos_words = pos_file.readlines()
            words.extend(pos_words)

    return [word.rstrip() for word in words]



from string import ascii_letters, digits
from random import choice
from typing import Optional, List, Union


def sequence(count: Optional[int] = None,
             length: int = 8) -> Union[str, List[str]]:
    '''
    Returns a random sequence consisting of ASCII symbols and digits
    or a list of a random sequences

    Args:
        count (int, optional): The number of sequances to be
            generated. Defaults to `None`
        length (int, optional): The length of the sequence.
            Defaults to `8`

    Returns:
        Union[str, List[str]]: The sequence if `count` is `None` or a list of sequences if `count` is not `None`
    '''
    chars = ascii_letters + digits

    if count:
        sequences = []
        for _ in range(count):
            sequences.append(''.join(choice(chars) for _ in range(length)))
        return sequences
    else:
        return ''.join(choice(chars) for _ in range(length))


def letter(count: Optional[int] = None) -> Union[str, List[str]]:
    '''
    Returns a random ASCII letter or a list of them

    Args:
        count (int, optional): The number of letters to be
            generated. Defaults to `None`

    Returns:
        Union[str, List[str]]: An ASCII letter if `count` is `None` or a list of letters if `count` is not `None`
    '''
    if count:
        letters = []
        for _ in range(count):
            letters.append(choice(ascii_letters))
        return letters
    else:
        return choice(ascii_letters)


def digit(count: Optional[int] = None) -> Union[str, List[str]]:
    '''
    Returns a random digit

    Args:
        count (int, optional): The number of digits to be
            generated. Defaults to `None`

    Returns:
        Union[str, List[str]]: A single digit if `count` is `None` or a list of digits if `count` is not `None`
    '''
    if count:
        digits_lst = []
        for _ in range(count):
            digits_lst.append(choice(digits))
        return digits_lst
    else:
        return choice(digits)


if __name__ == '__main__':
    print(sequence())
    print(letter())
    print(digit())



from random import choice, sample
from typing import Optional, List, Union

from .utilities import get_data


PARTS_OF_SPEECH = ['adj', 'adv', 'conj', 'interj', 'noun', 'prep', 'pron', 'verb']


def word(count: Optional[int] = None,
         include_pos: Optional[List[str]] = None,
         exclude_pos: Optional[List[str]] = None,
         word_len: Optional[int] = None,
         min_word_len: int = 1,
         max_word_len: Optional[int] = None,
         starts_with: Optional[str] = None,
         ends_with: Optional[str] = None,
         pattern: Optional[str] = None) -> Union[str, List[str]]:
    '''
    Returns a random English word or a list of words

    Abbreviation "pos" means "part of speech"

    Args:
        count (int, optional): The number of words to
            be generated. Defaults to None.
        include_pos (list of str, optional): List of parts of speech that will be
            included in the generation. Defaults to `None`
        exclude_pos (list of str, optional): List of parts of speech that will be
            excluded in the generation. Defaults to `None`
        word_len (int, optional): Specifies the length of a word. Ignores the
            `min_word_len` and `max_word_len` parameters. Defaults to `None`
        min_word_len (int, optional): The minimum word length. Defaults to `1`
        max_word_len (int, optional): The maximum word length. Defaults to `None`
        starts_with (str, optional): The pattern with which
            the word begins. Defaults to `None`
        ends_with (str, optional): The pattern with which
            the word ends. Defaults to `None`
        pattern (str, optional): The pattern that should be
            contained in the word. Defaults to `None`

    Returns:
        Union[str, List[str]]: A random English word if `count` is `None` or a list of them if `count` is not `None`

    Raises:
        IndexError: If the word was not found or if the
            desired number of words was not found
    '''
    if not include_pos:
        include_pos = PARTS_OF_SPEECH

    if exclude_pos:
        parts_of_speech = list(set(include_pos) - set(exclude_pos))
    else:
        parts_of_speech = include_pos

    words = []
    for part_of_speech in parts_of_speech:
        pos_words = get_data('parts_of_speech', part_of_speech)
        words.extend(pos_words)

    if max_word_len:
        filtered_words = list(filter(lambda word: min_word_len <= len(word) <= max_word_len, words))
    else:
        filtered_words = list(filter(lambda word: min_word_len <= len(word), words))

    if word_len:
        filtered_words = list(filter(lambda word: word_len == len(word), words))

    if starts_with:
        filtered_words = list(filter(lambda word: word.startswith(starts_with), filtered_words))
    if ends_with:
        filtered_words = list(filter(lambda word: word.endswith(ends_with), filtered_words))
    if pattern:
        filtered_words = list(filter(lambda word: pattern in word, filtered_words))

    if count:
        INDEX_ERROR_DESCRIPTION = 'The desired number of words was not found'

        if not filtered_words:
            raise IndexError(INDEX_ERROR_DESCRIPTION)

        try:
            final_words = sample(filtered_words, count)
        except ValueError:
            raise IndexError(INDEX_ERROR_DESCRIPTION)

        return final_words
    else:
        if not filtered_words:
            raise IndexError('The word was not found')

        return choice(filtered_words)


if __name__ == '__main__':
    print(word())



from random import choice, sample
from typing import Optional, List, Union

from .utilities import get_data


def country(count: Optional[int] = None) -> Union[str, List[str]]:
    '''
    Returns a random country or a list of a random countries

    Args:
        count (int, optional): The number of countries to be
            generated. Defaults to `None`

    Returns:
        Union[str, List[str]]: A random country if `count` is `None` or a list of countries if `count` is not `None`
    '''
    countries = get_data('places', 'countries')

    if count:
        return sample(countries, count)
    else:
        return choice(countries)


def city(count: Optional[int] = None) -> Union[str, List[str]]:
    '''
    Returns a random city or a list of them

    Args:
        count (int, optional): The number of cities to be
            generated. Defaults to `None`

    Returns:
        Union[str, List[str]]: A random city if `count` is `None` or a list of cities if `count` is not `None`
    '''
    cities = get_data('places', 'cities')

    if count:
        return sample(cities, count)
    else:
        return choice(cities)


if __name__ == '__main__':
    print(country())
    print(city())



import argparse

from randword import (
    word,
    name,
    surname,
    fullname,
    sequence,
    letter,
    digit,
    country,
    city,
    magic_8ball,
    flip_coin,
)


def main():
    parser = argparse.ArgumentParser(description="CLI for randword package")
    subparsers = parser.add_subparsers(dest="command", required=True)

    # === word ===
    word_parser = subparsers.add_parser("word", help="Generate random word(s)")
    word_parser.add_argument(
        "count", nargs="?", type=int, default=None, help="Number of words to generate"
    )
    word_parser.add_argument(
        "-i",
        "--include_pos",
        nargs="+",
        help="Parts of speech to include (e.g. adj verb)",
    )
    word_parser.add_argument(
        "-e", "--exclude_pos", nargs="+", help="Parts of speech to exclude (e.g. noun)"
    )
    word_parser.add_argument(
        "-min", "--min_word_len", type=int, help="Minimum word length"
    )
    word_parser.add_argument(
        "-max", "--max_word_len", type=int, help="Maximum word length"
    )
    word_parser.add_argument("-l", "--word_len", type=int, help="Exact word length")
    word_parser.add_argument(
        "-s", "--starts_with", help="String the word should start with"
    )
    word_parser.add_argument(
        "-x", "--ends_with", help="String the word should end with"
    )
    word_parser.add_argument("-p", "--pattern", help="Pattern the word should contain")

    # === name ===
    name_parser = subparsers.add_parser("name", help="Generate random name(s)")
    name_parser.add_argument(
        "count", nargs="?", type=int, default=None, help="Number of names to generate"
    )
    name_parser.add_argument(
        "-g", "--gender", choices=["m", "f"], help="Gender of name (m or f)"
    )

    # === surname ===
    surname_parser = subparsers.add_parser("surname", help="Generate random surname(s)")
    surname_parser.add_argument(
        "count",
        nargs="?",
        type=int,
        default=None,
        help="Number of surnames to generate",
    )

    # === fullname ===
    fullname_parser = subparsers.add_parser(
        "fullname", help="Generate random full name(s)"
    )
    fullname_parser.add_argument(
        "count",
        nargs="?",
        type=int,
        default=None,
        help="Number of full names to generate",
    )
    fullname_parser.add_argument(
        "-g", "--gender", choices=["m", "f"], help="Gender of full name (m or f)"
    )

    # === sequence ===
    sequence_parser = subparsers.add_parser(
        "sequence", help="Generate random alphanumeric sequence(s)"
    )
    sequence_parser.add_argument(
        "count",
        nargs="?",
        type=int,
        default=None,
        help="Number of sequences to generate",
    )
    sequence_parser.add_argument(
        "length", nargs="?", type=int, default=None, help="Length of each sequence"
    )

    # === letter ===
    letter_parser = subparsers.add_parser("letter", help="Generate random letter(s)")
    letter_parser.add_argument(
        "count", nargs="?", type=int, default=None, help="Number of letters to generate"
    )

    # === digit ===
    digit_parser = subparsers.add_parser("digit", help="Generate random digit(s)")
    digit_parser.add_argument(
        "count", nargs="?", type=int, default=None, help="Number of digits to generate"
    )

    # === country ===
    country_parser = subparsers.add_parser(
        "country", help="Generate random country/countries"
    )
    country_parser.add_argument(
        "count",
        nargs="?",
        type=int,
        default=None,
        help="Number of countries to generate",
    )

    # === city ===
    city_parser = subparsers.add_parser("city", help="Generate random city/cities")
    city_parser.add_argument(
        "count", nargs="?", type=int, default=None, help="Number of cities to generate"
    )

    # === magic_8ball ===
    subparsers.add_parser("magic_8ball", help="Ask the Magic 8-Ball a question")

    # === flip_coin ===
    subparsers.add_parser("flip_coin", help="Flip a coin (True=Heads, False=Tails)")

    # Default to "word" if no subcommand is given
    # if len(sys.argv) == 1 or sys.argv[1].startswith("-"):
    #     sys.argv.insert(1, "word")

    args = parser.parse_args()

    # === Dispatch Logic ===
    if args.command == "word":
        kwargs = {
            "include_pos": args.include_pos,
            "exclude_pos": args.exclude_pos,
            "min_word_len": args.min_word_len,
            "max_word_len": args.max_word_len,
            "word_len": args.word_len,
            "starts_with": args.starts_with,
            "ends_with": args.ends_with,
            "pattern": args.pattern,
        }
        kwargs = {k: v for k, v in kwargs.items() if v is not None}
        result = (
            word(args.count, **kwargs) if args.count is not None else word(**kwargs)
        )

    elif args.command == "name":
        result = (
            name(args.count, args.gender)
            if args.count is not None
            else name(gender=args.gender)
        )

    elif args.command == "surname":
        result = surname(args.count) if args.count else surname()

    elif args.command == "fullname":
        result = (
            fullname(args.count, args.gender)
            if args.count is not None
            else fullname(gender=args.gender)
        )

    elif args.command == "sequence":
        if args.count is not None and args.length is not None:
            result = sequence(args.count, args.length)
        elif args.count is not None:
            result = sequence(args.count)
        else:
            result = sequence()

    elif args.command == "letter":
        result = letter(args.count) if args.count else letter()

    elif args.command == "digit":
        result = digit(args.count) if args.count else digit()

    elif args.command == "country":
        result = country(args.count) if args.count else country()

    elif args.command == "city":
        result = city(args.count) if args.count else city()

    elif args.command == "magic_8ball":
        _ = magic_8ball()
        return

    elif args.command == "flip_coin":
        result = flip_coin()

    print(result)


if __name__ == "__main__":
    main()
